@@ -9,8 +9,8 @@
 
 ;; Author: Chris Done <chrisdone@gmail.com>
 ;; URL: https://github.com/emacsorphanage/god-mode
-;; Version: 2.18.0
-;; Package-Requires: ((emacs "25.1"))
+;; Version: 2.19.0
+;; Package-Requires: ((emacs "26.3"))
 
 ;; This file is not part of GNU Emacs.
 
@@ -109,6 +109,18 @@ in God mode will be translated to `C-<f5>'."
   :group 'god
   :type 'boolean)
 
+(defcustom god-mode-lighter-string
+  "God"
+  "String displayed on the mode line when God mode is active.
+Set it to nil if you don't want a mode line indicator."
+  :group 'god
+  :type '(choice string (const :tag "None" nil)))
+
+(defface god-mode-lighter
+  '((t))
+  "Face for God mode's lighter."
+  :group 'god)
+
 (defun god-mode-make-f-key (n &optional shift)
   "Get the event for numbered function key N, with shift status SHIFT.
 For example, calling with arguments 5 and t yields the symbol `S-f5'."
@@ -134,7 +146,9 @@ For example, calling with arguments 5 and t yields the symbol `S-f5'."
 (define-minor-mode god-local-mode
   "Minor mode for running commands."
   :init-value nil
-  :lighter " God"
+  :lighter (god-mode-lighter-string
+            ((" "
+             (:propertize god-mode-lighter-string face god-mode-lighter))))
   :keymap god-local-mode-map
   (if god-local-mode
       (run-hooks 'god-mode-enabled-hook)
@@ -158,7 +172,7 @@ If it was not active when `god-local-mode-pause' was called, nothing happens."
   "Enable `god-local-mode' on all buffers.")
 
 (defvar god-literal-sequence nil
-  "Activated after `god-literal-key' is pressed in a command sequence.")
+  "Toggled when `god-literal-key' is pressed in a command sequence.")
 
 ;;;###autoload
 (defun god-mode ()
@@ -170,10 +184,18 @@ If it was not active when `god-local-mode-pause' was called, nothing happens."
     (god-local-mode -1)))
 
 ;;;###autoload
-(defun god-mode-all ()
-  "Toggle `god-local-mode' in all buffers."
+(defun god-mode-all (&optional arg)
+  "Toggle `god-local-mode' in all buffers.
+
+Toggle the mode if ARG is nil. If ARG is non-nil, enable the mode
+if ARG is zero or a positive number, or disable the mode if ARG
+is a negative number."
   (interactive)
-  (let ((new-status (if (bound-and-true-p god-local-mode) -1 1)))
+  (let ((new-status
+	 (cond
+	  ((null arg) (if (bound-and-true-p god-local-mode) -1 1))
+	  ((> 0 arg) -1)
+	  (t 1))))
     (setq god-global-mode t)
     (mapc (lambda (buffer)
             (with-current-buffer buffer
@@ -286,51 +308,76 @@ from the command loop."
   "Interpret god-mode special keys for KEY.
 Consumes more keys if appropriate.
 Appends to key sequence KEY-STRING-SO-FAR."
-  (let ((key-consumed t) (next-modifier "") next-key)
+  (let ((modifier "")
+        (next-key (lambda () (god-mode-sanitized-key-string (read-event key-string-so-far)))))
     (message key-string-so-far)
-    (cond
-     ;; Don't check for `god-literal-key' with the first key.
-     ((and key-string-so-far (string= key god-literal-key))
-      (setq god-literal-sequence t))
-     (god-literal-sequence
-      (setq key-consumed nil))
-     ((and (stringp key) (assoc key god-mode-alist))
-      (setq next-modifier (cdr (assoc key god-mode-alist))))
-     (t
-      (setq key-consumed nil
-            next-modifier (cdr (assoc nil god-mode-alist)))))
-    (setq next-key
-          (if key-consumed
-              (god-mode-sanitized-key-string (read-event key-string-so-far))
-            key))
-    (if (and key-string-so-far (string= next-key (format "%c" help-char)))
-        (god-mode-help-char-dispatch next-key key-string-so-far)
-      (when (and (= (length next-key) 1)
-                 (string= (get-char-code-property (aref next-key 0) 'general-category) "Lu")
+    (when key-string-so-far ; Don't check for `god-literal-key' with the first key.
+      (while (string= key god-literal-key)
+        (setq god-literal-sequence (not god-literal-sequence)
+              key (funcall next-key))))
+    (unless god-literal-sequence
+      (let ((modifier-lookup (and (stringp key) (assoc key god-mode-alist))))
+        (if modifier-lookup
+            (setq modifier (cdr modifier-lookup)
+                  key (funcall next-key))
+          (setq modifier (cdr (assoc nil god-mode-alist))))))
+    (if (and key-string-so-far (string= key (format "%c" help-char)))
+        (god-mode-help-char-dispatch key key-string-so-far)
+      (when (and (= (length key) 1)
+                 (string= (get-char-code-property (aref key 0) 'general-category) "Lu")
                  ;; If C- is part of the modifier, S- needs to be given
                  ;; in order to distinguish the uppercase from the
                  ;; lowercase bindings. If C- is not in the modifier,
                  ;; then emacs natively treats uppercase differently
                  ;; from lowercase, and the S- modifier should not be
                  ;; supplied.
-                 (string-prefix-p "C-" next-modifier))
-        (setq next-modifier (concat next-modifier "S-")))
+                 (string-prefix-p "C-" modifier))
+        (setq modifier (concat modifier "S-")))
       (if key-string-so-far
-          (concat key-string-so-far " " next-modifier next-key)
-        (concat next-modifier next-key)))))
+          (concat key-string-so-far " " modifier key)
+        (concat modifier key)))))
 
 (defun god-mode-lookup-command (key-string)
-  "Execute extended keymaps in KEY-STRING, or call it if it is a command."
+  "Look up and return the command for KEY-STRING.
+Tries fallback for the last key by removing Control modifier. If no command is found,
+returns a keymap to allow further key input, or nil if completely unbound."
   (when key-string
     (let* ((key-vector (read-kbd-macro key-string t))
            (binding (key-binding key-vector)))
-      (cond ((commandp binding)
-             (setq last-command-event (aref key-vector (- (length key-vector) 1)))
-             binding)
-            ((keymapp binding)
-             (god-mode-lookup-key-sequence nil key-string))
-            (:else
-             (error "God: Unknown key binding for `%s`" key-string))))))
+      (cond
+       ;; Case 1: found an actual command
+       ((commandp binding)
+        (setq last-command-event (aref key-vector (1- (length key-vector))))
+        binding)
+
+       ;; Case 2: it's a keymap (partial key sequence)
+       ((keymapp binding)
+        (god-mode-lookup-key-sequence nil key-string))
+
+       ;; Case 3: try fallback by removing C- from the last key
+       ((string-match "\\(.*\\) C-\\(.\\)$" key-string)
+        (let* ((prefix (match-string 1 key-string))
+               (last (match-string 2 key-string))
+               (key-str (format "%s %s" prefix last))
+               (fallback-key (if (>= emacs-major-version 28)
+                                 (string-trim-right key-str)
+                               (replace-regexp-in-string
+                                "[ \t\n\r]+$" "" ;; Remove trailing whitespace
+                                key-str)))
+               (fallback-vector (read-kbd-macro fallback-key t))
+               (fallback-binding (key-binding fallback-vector)))
+          (cond
+           ((commandp fallback-binding)
+            (message "Using fallback command: %s" fallback-key)
+            (setq last-command-event (aref fallback-vector (1- (length fallback-vector))))
+            fallback-binding)
+
+           ((keymapp fallback-binding)
+            (message "Using fallback keymap: %s" fallback-key)
+            (god-mode-lookup-key-sequence nil fallback-key))
+
+           ;; Not a command nor a map — wait for more input, don't fail
+           )))))))
 
 ;;;###autoload
 (defun god-mode-maybe-activate (&optional status)
